package com.test.testsaml;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.security.Key;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMValidateContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class Test {
   public static String test() throws Exception {
      String certificateString = readCertificate();
      InputStream stream  = Test.class.getResourceAsStream("test.xml");
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(true);
      DocumentBuilder builder = factory.newDocumentBuilder();
      InputSource source = new InputSource(stream);
      Document document = builder.parse(source);
      NodeList signatureNodes = document.getDocumentElement().getElementsByTagNameNS(XMLSignature.XMLNS, "Signature");
      if(signatureNodes.getLength() == 0) {
         throw new Exception("Cannot find Signature element");
      }
      return checkSignature(certificateString, signatureNodes);
   }
   
   private static String readCertificate() throws Exception {
      BufferedReader reader = new BufferedReader(new InputStreamReader(Test.class.getResourceAsStream("certificate.txt"), "UTF-8"));
      StringBuilder builder = new StringBuilder();
      String line = reader.readLine();
      while(line != null) {
         builder.append(line);
         line = reader.readLine();
         if(line != null) {
            builder.append("\n");
         }
      }
      return builder.toString();
   }
   
   private static String checkSignature(String samlCertificate, NodeList signatureNodes) {
      Key key = null;
      CertificateFactory certificateFactory;
      try {
         certificateFactory = CertificateFactory.getInstance("X.509");
         if(samlCertificate == null) {
            throw new RuntimeException("No SAML certificate");
         }
         Certificate certificate = certificateFactory.generateCertificate(new ByteArrayInputStream(samlCertificate.getBytes()));
         if(certificate instanceof X509Certificate) {
            X509Certificate xCertificate = (X509Certificate)certificate;
            key = xCertificate.getPublicKey();
         } else {
            throw new RuntimeException("SAML certificate is not a valid X.509 certificate");
         }
      } catch(CertificateException exc) {
         throw new RuntimeException("Error initializing certificate during signature check", exc);
      }
      try {
         DOMValidateContext validateContext = new DOMValidateContext(key, signatureNodes.item(0));
         XMLSignatureFactory fac = XMLSignatureFactory.getInstance("DOM");
         XMLSignature signature2 = fac.unmarshalXMLSignature(validateContext);

         boolean coreValidity = signature2.validate(validateContext);
         if(!coreValidity) {
            return "Invalid signature in SAML Response";
         }
         return null;
      } catch(MarshalException exc) {
         throw new RuntimeException("Error processing SAML signature", exc);
      } catch(XMLSignatureException exc) {
         throw new RuntimeException("Error processing SAML signature", exc);
      }
   }

   public static void main(String[] args) throws Exception {
      String result = test();
      if(result == null) {
         System.err.println("OK");
      } else {
         System.err.println("Validation Failed: "+result);
      }
   }
}
